// -*- mode: c++ -*-

#ifndef AST_H
#define AST_H

#include <vector>
#include <string>

enum {
  op_not,
  op_neg,
  op_deref,
  op_ref,

  op_add,
  op_sub,
  op_mul,
  op_div,
  op_assign,
  op_or,
  op_and,
  op_eq,
  op_not_eq,
  op_lt,
  op_gt,
  op_le,
  op_ge
};

class AST;
class ArgumentAST;
class ArgumentsAST;
class ArrayDeclaratorAST;
class BinaryExpressionAST;
class BlockAST;
class CallExpressionAST;
class DeclarationAST;
class DeclarationsAST;
class DeclaratorAST;
class DeclaratorsAST;
class ExpressionAST;
class ExpressionListAST;
class ExpressionStatementAST;
class FunctionAST;
class FunctionDeclaratorAST;
class IdentifierAST;
class StringLiteralAST;
class IdentifierDeclaratorAST;
class IfStatementAST;
class IntConstantAST;
class PointerAST;
class ReturnStatementAST;
class StatementAST;
class StatementsAST;
class SubDeclaratorAST;
class TranslationUnitAST;
class UnaryExpressionAST;
class WhileStatementAST;
class CastExpressionAST;

class ASTVisitor {
public:
  virtual ~ASTVisitor() {}

  virtual bool preVisit(AST *) { return true; }
  virtual void postVisit(AST *) {}

  virtual bool visit(CastExpressionAST *) { return true; }
  virtual bool visit(ArgumentAST *) { return true; }
  virtual bool visit(ArgumentsAST *) { return true; }
  virtual bool visit(ArrayDeclaratorAST *) { return true; }
  virtual bool visit(BinaryExpressionAST *) { return true; }
  virtual bool visit(BlockAST *) { return true; }
  virtual bool visit(CallExpressionAST *) { return true; }
  virtual bool visit(DeclarationAST *) { return true; }
  virtual bool visit(DeclarationsAST *) { return true; }
  virtual bool visit(DeclaratorAST *) { return true; }
  virtual bool visit(DeclaratorsAST *) { return true; }
  virtual bool visit(ExpressionAST *) { return true; }
  virtual bool visit(ExpressionListAST *) { return true; }
  virtual bool visit(ExpressionStatementAST *) { return true; }
  virtual bool visit(FunctionAST *) { return true; }
  virtual bool visit(FunctionDeclaratorAST *) { return true; }
  virtual bool visit(IdentifierAST *) { return true; }
  virtual bool visit(IdentifierDeclaratorAST *) { return true; }
  virtual bool visit(IfStatementAST *) { return true; }
  virtual bool visit(IntConstantAST *) { return true; }
  virtual bool visit(PointerAST *) { return true; }
  virtual bool visit(ReturnStatementAST *) { return true; }
  virtual bool visit(StatementAST *) { return true; }
  virtual bool visit(StatementsAST *) { return true; }
  virtual bool visit(SubDeclaratorAST *) { return true; }
  virtual bool visit(UnaryExpressionAST *) { return true; }
  virtual bool visit(WhileStatementAST *) { return true; }
  virtual bool visit(TranslationUnitAST *) { return true; }
  virtual bool visit(StringLiteralAST *) { return true; }

  virtual void endVisit(ArgumentAST *) {}
  virtual void endVisit(ArgumentsAST *) {}
  virtual void endVisit(ArrayDeclaratorAST *) {}
  virtual void endVisit(BinaryExpressionAST *) {}
  virtual void endVisit(BlockAST *) {}
  virtual void endVisit(CallExpressionAST *) {}
  virtual void endVisit(DeclarationAST *) {}
  virtual void endVisit(DeclarationsAST *) {}
  virtual void endVisit(DeclaratorAST *) {}
  virtual void endVisit(DeclaratorsAST *) {}
  virtual void endVisit(ExpressionAST *) {}
  virtual void endVisit(ExpressionListAST *) {}
  virtual void endVisit(ExpressionStatementAST *) {}
  virtual void endVisit(FunctionAST *) {}
  virtual void endVisit(FunctionDeclaratorAST *) {}
  virtual void endVisit(IdentifierAST *) {}
  virtual void endVisit(IdentifierDeclaratorAST *) {}
  virtual void endVisit(IfStatementAST *) {}
  virtual void endVisit(IntConstantAST *) {}
  virtual void endVisit(PointerAST *) {}
  virtual void endVisit(ReturnStatementAST *) {}
  virtual void endVisit(StatementAST *) {}
  virtual void endVisit(StatementsAST *) {}
  virtual void endVisit(SubDeclaratorAST *) {}
  virtual void endVisit(UnaryExpressionAST *) {}
  virtual void endVisit(WhileStatementAST *) {}
  virtual void endVisit(TranslationUnitAST *) {}
  virtual void endVisit(StringLiteralAST *) {}
  virtual void endVisit(CastExpressionAST *) {}
};

class AST
{
public:
  virtual ~AST() {}

  virtual void accept(ASTVisitor *visitor) {
    if (visitor->preVisit(this))
      accept0(visitor);
    visitor->postVisit(this);
  }

  static void acceptNode(AST *node, ASTVisitor *visitor) {
    if (! node)
      return;

    node->accept(visitor);
  }

  template <typename _ForwardIterator>
  static void acceptNodeList(_ForwardIterator first, _ForwardIterator last, ASTVisitor *visitor) {
    for (; first != last; ++first)
      (*first)->accept(visitor);
  }

protected:
  virtual void accept0(ASTVisitor *visitor) = 0;
};

class StatementAST: public AST {
protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
    }
    visitor->endVisit(this);
  }
};

class StatementsAST: public AST, public std::vector<StatementAST*>
{
protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNodeList(begin(), end(), visitor);
    }
    visitor->endVisit(this);
  }
};

class DeclaratorAST: public AST {
protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
    }
    visitor->endVisit(this);
  }
};

class DeclaratorsAST: public AST, public std::vector<DeclaratorAST*>
{
protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNodeList(begin(), end(), visitor);
    }
    visitor->endVisit(this);
  }
};

class DeclarationAST: public AST {
public:
  const std::string *type_specifier;
  DeclaratorsAST *declarators;

protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNode(declarators, visitor);
    }
    visitor->endVisit(this);
  }
};

class DeclarationsAST: public AST, public std::vector<DeclarationAST*>
{
protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNodeList(begin(), end(), visitor);
    }
    visitor->endVisit(this);
  }
};

class BlockAST: public StatementAST {
public:
  DeclarationsAST *declarations;
  StatementsAST *statements;

protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNode(declarations, visitor);
      acceptNode(statements, visitor);
    }
    visitor->endVisit(this);
  }
};

class ArgumentAST: public AST {
public:
  const std::string *type_specifier;
  DeclaratorAST *declarator;

protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNode(declarator, visitor);
    }
    visitor->endVisit(this);
  }
};

class ArgumentsAST: public AST, public std::vector<ArgumentAST*>
{
protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
    }
    visitor->endVisit(this);
  }
};

class FunctionAST: public AST {
public:
  const std::string *type_specifier;
  DeclaratorAST *declarator;
  BlockAST *block;

protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNode(declarator, visitor);
      acceptNode(block, visitor);
    }
    visitor->endVisit(this);
  }
};

class ExpressionAST: public AST {
protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
    }
    visitor->endVisit(this);
  }
};

class ExpressionStatementAST: public StatementAST {
public:
  ExpressionAST *expression;

protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNode(expression, visitor);
    }
    visitor->endVisit(this);
  }
};

class ReturnStatementAST: public StatementAST {
public:
  ExpressionAST *expression;

protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNode(expression, visitor);
    }
    visitor->endVisit(this);
  }
};

class IfStatementAST: public StatementAST {
public:
  ExpressionAST *condition;
  StatementAST *then_statement;
  StatementAST *else_statement;

protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNode(condition, visitor);
      acceptNode(then_statement, visitor);
      acceptNode(else_statement, visitor);
    }
    visitor->endVisit(this);
  }
};

class WhileStatementAST: public StatementAST {
public:
  ExpressionAST *condition;
  StatementAST *statement;

protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNode(condition, visitor);
      acceptNode(statement, visitor);
    }
    visitor->endVisit(this);
  }
};

class PointerAST: public DeclaratorAST {
public:
  DeclaratorAST *declarator;

protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNode(declarator,  visitor);
    }
    visitor->endVisit(this);
  }
};

class IdentifierDeclaratorAST: public DeclaratorAST {
public:
  const std::string *id;

protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
    }
    visitor->endVisit(this);
  }
};

class FunctionDeclaratorAST: public DeclaratorAST {
public:
  DeclaratorAST *declarator;
  ArgumentsAST *arguments;

protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNode(declarator, visitor);
      acceptNode(arguments, visitor);
    }
    visitor->endVisit(this);
  }
};

class ArrayDeclaratorAST: public DeclaratorAST {
public:
  DeclaratorAST *declarator;
  int dimension;

protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNode(declarator, visitor);
    }
    visitor->endVisit(this);
  }
};

class SubDeclaratorAST: public DeclaratorAST {
public:
  DeclaratorAST *declarator;

protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNode(declarator, visitor);
    }
    visitor->endVisit(this);
  }
};

class ExpressionListAST: public AST, public std::vector<ExpressionAST*>
{
protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNodeList(begin(), end(), visitor);
    }
    visitor->endVisit(this);
  }
};

class IdentifierAST: public ExpressionAST {
public:
  const std::string *id;

protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
    }
    visitor->endVisit(this);
  }
};

class StringLiteralAST: public ExpressionAST {
public:
  const std::string *value;

protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
    }
    visitor->endVisit(this);
  }
};

class IntConstantAST: public ExpressionAST {
public:
  int value;

protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
    }
    visitor->endVisit(this);
  }
};

class UnaryExpressionAST: public ExpressionAST {
public:
  UnaryExpressionAST(int op, ExpressionAST *expression):
    op(op), expression(expression) {}

  int op;
  ExpressionAST *expression;

protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNode(expression, visitor);
    }
    visitor->endVisit(this);
  }
};

class BinaryExpressionAST: public ExpressionAST {
public:
  BinaryExpressionAST(int op, ExpressionAST *left_expression, ExpressionAST *right_expression):
    op(op), left_expression(left_expression), right_expression(right_expression) {}

  int op;
  ExpressionAST *left_expression;
  ExpressionAST *right_expression;

protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNode(left_expression, visitor);
      acceptNode(right_expression, visitor);
    }
    visitor->endVisit(this);
  }
};

class CallExpressionAST: public ExpressionAST {
public:
  ExpressionAST *expression;
  ExpressionListAST *expression_list;

protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNode(expression, visitor);
      acceptNode(expression_list, visitor);
    }
    visitor->endVisit(this);
  }
};

class CastExpressionAST: public ExpressionAST {
public:
  const std::string *type_specifier;
  DeclaratorAST *declarator;
  ExpressionAST *expression;

protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNode(declarator, visitor);
      acceptNode(expression, visitor);
    }
    visitor->endVisit(this);
  }
};

class TranslationUnitAST: public AST, public std::vector<AST*>
{
protected:
  virtual void accept0(ASTVisitor *visitor) {
    if (visitor->visit(this)) {
      acceptNodeList(begin(), end(), visitor);
    }
    visitor->endVisit(this);
  }
};

#endif // AST_H

