
%merged_output parser.cpp
%parser T

%expect 1

%token T_AND T_AND_AND T_CHAR T_COMMA
%token T_DIV T_ELSE T_ASSIGN T_EQ_EQ T_GE T_GT
%token T_ID T_IF T_INT T_LBRACE T_LBRACKET
%token T_LE T_LPAREN T_LT T_MINUS T_NOT
%token T_NOT_EQ T_NUM T_OR_OR T_PLUS T_RBRACE
%token T_RBRACKET T_RETURN T_RPAREN T_SEMICOLON T_STAR
%token T_STRING_LITERAL T_WHILE

%right T_ASSIGN
%left T_OR_OR
%left T_AND_AND
%left T_EQ_EQ T_NOT_EQ
%left T_LT T_GT T_LE T_GE
%left T_PLUS T_MINUS
%left T_STAR T_DIV
%right uminus_prec
%left T_LPAREN

%start TranslationUnit

/.

#include <string>
#include <set>
#include <iostream>

#include "ast.h"

union Value {
  int int_ast;
  const std::string *string_ast;
  AST *ast_ast;
  ArgumentAST *argument_ast;
  ArgumentsAST *arguments_ast;
  ArrayDeclaratorAST *array_declarator_ast;
  BinaryExpressionAST *binary_expression_ast;
  BlockAST *block_ast;
  CallExpressionAST *call_expression_ast;
  CastExpressionAST *cast_expression_ast;
  DeclarationAST *declaration_ast;
  DeclarationsAST *declarations_ast;
  DeclaratorAST *declarator_ast;
  DeclaratorsAST *declarators_ast;
  ExpressionAST *expression_ast;
  ExpressionListAST *expression_list_ast;
  ExpressionStatementAST *expression_statement_ast;
  FunctionAST *function_ast;
  FunctionDeclaratorAST *function_declarator_ast;
  IdentifierAST *identifier_ast;
  IdentifierDeclaratorAST *identifier_declarator_ast;
  IfStatementAST *if_statement_ast;
  IntConstantAST *int_constant_ast;
  PointerAST *pointer_ast;
  ReturnStatementAST *return_statement_ast;
  StatementAST *statement_ast;
  StatementsAST *statements_ast;
  StringLiteralAST *string_literal_ast;
  SubDeclaratorAST *sub_declarator_ast;
  TranslationUnitAST *translation_unit_ast;
  UnaryExpressionAST *unary_expression_ast;
  WhileStatementAST *while_statement_ast;
};

TranslationUnitAST *translation_unit;

const std::string *intern(const std::string &s)
{
  static std::set<std::string> string_set;

  return &*string_set.insert(s).first;
}

class MiniCParser: $table
{
  int state_stack[128];
  Value sym_stack[128];
  int tos;
  Value yylval;

  int yylineno;
  std::string yyfilename;
  
  const std::string *escape(const std::string &s)
  {
    std::string r;
    
    for (size_t i = 0; i < s.size(); ++i)
      {
	const char ch = s[i];
	
	if (ch != '\\')
	  r.push_back(ch);
	else
	  {
	    assert(i + 1 < s.length());
	    
	    switch (s[++i]) {
	    case 'n': r.push_back('\n'); break;
	    case 't': r.push_back('\t'); break;
	    case 'r': r.push_back('\r'); break;
	    case 'b': r.push_back('\b'); break;
	    case '\\': r.push_back('\\'); break;
	    case '\'': r.push_back('\''); break;
	    case '"': r.push_back('"'); break;
	    default:
	      std::cerr << "*** Unknown Escape character" << std::endl;
	      exit(EXIT_FAILURE);
	    }
	  }
      }

    return intern(r);
  }

  inline Value &sym(int index) { return sym_stack[tos + index - 1]; }

public:
  MiniCParser(const std::string &yyfilename):
    yyfilename(yyfilename), yylineno(1) {}

  int yylex();

  TranslationUnitAST *parse() {
    int act = 0;
    int token = -1;

    translation_unit = 0;
    tos = 0;
    state_stack[tos] = 0;
    sym_stack[tos].ast_ast = 0;

    for(;;)
      {
	if (token == -1 && -TERMINAL_COUNT != action_index[act])
	  token = yylex();

	act = t_action(act, token);
	
	if (act > 0)
	  {
	    if (act == ACCEPT_STATE)
	      {
		assert(!--tos);

		return translation_unit;
	      }

	    ++tos;
	    state_stack[tos] = act;
	    sym_stack[tos] = yylval;
	    token = -1;
	  }

	else if (act < 0)
	  {
	    const int r = -act - 1;

	    tos -= rhs[r];

	    act = nt_action(state_stack[tos], lhs[r] - TERMINAL_COUNT);

	    state_stack[++tos] = act;

switch(r) {
./



TranslationUnit ::= ExternalDefinition ;
/.
case $rule_number: {
  TranslationUnitAST *ast = new TranslationUnitAST;
  ast->push_back(sym(1).ast_ast);
  sym(1).translation_unit_ast = ast;
  translation_unit = ast;
} break;
./

TranslationUnit ::= TranslationUnit ExternalDefinition ;
/.
case $rule_number: {
  sym(1).translation_unit_ast->push_back(sym(2).ast_ast);
} break;
./

ExternalDefinition ::= FunctionDefinition ;
ExternalDefinition ::= Declaration ;

FunctionDefinition ::= TypeSpec Declarator Block ;
/.
case $rule_number: {
  FunctionAST *ast = new FunctionAST;
  ast->type_specifier = sym(1).string_ast;
  ast->declarator = sym(2).declarator_ast;
  ast->block = sym(3).block_ast;
  sym(1).function_ast = ast;
} break;
./

Declaration ::= TypeSpec Declarators T_SEMICOLON ;
/.
case $rule_number: {
  DeclarationAST *ast = new DeclarationAST;
  ast->type_specifier = sym(1).string_ast;
  ast->declarators = sym(2).declarators_ast;
  sym(1).declaration_ast = ast;
} break;
./

TypeSpec ::= T_INT ;
/.
case $rule_number: {
  sym(1).string_ast = intern("int");
} break;
./

TypeSpec ::= T_CHAR ;
/.
case $rule_number: {
  sym(1).string_ast = intern("char");
} break;
./

ArgumentsOpt ::= ;
/.
case $rule_number: {
  sym(1).arguments_ast = 0;
} break;
./

ArgumentsOpt ::= Arguments ;

Argument ::= TypeSpec Declarator ;
/.case $rule_number:./

Argument ::= TypeSpec AbstractDeclaratorOpt ;
/.
case $rule_number: {
  ArgumentAST *ast = new ArgumentAST;
  ast->type_specifier = sym(1).string_ast;
  ast->declarator = sym(2).declarator_ast;
  sym(1).argument_ast = ast;
} break;
./

Arguments ::= Argument ;
/.
case $rule_number: {
  ArgumentsAST *ast = new ArgumentsAST;
  ast->push_back(sym(1).argument_ast);
  sym(1).arguments_ast = ast;
} break;
./

Arguments ::= Arguments T_COMMA Argument ;
/.
case $rule_number: {
  sym(1).arguments_ast->push_back(sym(3).argument_ast);
} break;
./

Block ::= T_LBRACE DeclarationsOpt StatementsOpt T_RBRACE ;
/.
case $rule_number: {
  BlockAST *ast = new BlockAST;
  ast->declarations = sym(2).declarations_ast;
  ast->statements = sym(3).statements_ast;
  sym(1).block_ast = ast;
} break;
./

DeclarationsOpt ::= ;
/.
case $rule_number: {
  sym(1).declarations_ast = 0;
} break;
./

DeclarationsOpt ::= Declarations ;

Declarations ::= Declaration ;
/.
case $rule_number: {
  DeclarationsAST *ast = new DeclarationsAST;
  ast->push_back(sym(1).declaration_ast);
  sym(1).declarations_ast = ast;
} break;
./

Declarations ::= Declarations Declaration ;
/.
case $rule_number: {
  sym(1).declarations_ast->push_back(sym(2).declaration_ast);
} break;
./

StatementsOpt ::= ;
/.
case $rule_number: {
  sym(1).statements_ast = 0;
} break;
./

StatementsOpt ::= Statements ;

Statements ::= Statement ;
/.
case $rule_number: {
  StatementsAST *ast = new StatementsAST;
  ast->push_back(sym(1).statement_ast);
  sym(1).statements_ast = ast;
} break;
./

Statements ::= Statements Statement ;
/.
case $rule_number: {
  sym(1).statements_ast->push_back(sym(2).statement_ast);
} break;
./

Statement ::= Block ;

Statement ::= Expression T_SEMICOLON ;
/.
case $rule_number: {
  ExpressionStatementAST *ast = new ExpressionStatementAST;
  ast->expression = sym(1).expression_ast;
  sym(1).expression_statement_ast = ast;
} break;
./

Statement ::= T_SEMICOLON ;
/.
case $rule_number: {
  ExpressionStatementAST *ast = new ExpressionStatementAST;
  ast->expression = 0;
  sym(1).expression_statement_ast = ast;
} break;
./

Statement ::= T_IF T_LPAREN Expression T_RPAREN Statement ;
/.
case $rule_number: {
  IfStatementAST *ast = new IfStatementAST;
  ast->condition = sym(3).expression_ast;
  ast->then_statement = sym(5).statement_ast;
  sym(1).if_statement_ast = ast;
} break;
./

Statement ::= T_IF T_LPAREN Expression T_RPAREN Statement T_ELSE Statement ;
/.
case $rule_number: {
  IfStatementAST *ast = new IfStatementAST;
  ast->condition = sym(3).expression_ast;
  ast->then_statement = sym(5).statement_ast;
  ast->else_statement = sym(7).statement_ast;
  sym(1).if_statement_ast = ast;
} break;
./

Statement ::= T_WHILE T_LPAREN Expression T_RPAREN Statement ;
/.
case $rule_number: {
  WhileStatementAST *ast = new WhileStatementAST;
  ast->condition = sym(3).expression_ast;
  ast->statement = sym(5).statement_ast;
  sym(1).while_statement_ast = ast;
} break;
./

Statement ::= T_RETURN Expression T_SEMICOLON ;
/.
case $rule_number: {
  ReturnStatementAST *ast = new ReturnStatementAST;
  ast->expression = sym(2).expression_ast;
  sym(1).return_statement_ast = ast;
} break;
./

Statement ::= T_RETURN T_SEMICOLON ;
/.
case $rule_number: {
  ReturnStatementAST *ast = new ReturnStatementAST;
  ast->expression = 0;
  sym(1).return_statement_ast = ast;
} break;
./

Declarators ::= Declarator ;
/.
case $rule_number: {
  DeclaratorsAST *ast = new DeclaratorsAST;
  ast->push_back(sym(1).declarator_ast);
  sym(1).declarators_ast = ast;
} break;
./

Declarators ::= Declarators T_COMMA Declarator ;
/.
case $rule_number: {
  sym(1).declarators_ast->push_back(sym(3).declarator_ast);
} break;
./

Declarator ::= T_STAR Declarator ;
/.
case $rule_number: {
  PointerAST *ast = new PointerAST;
  ast->declarator = sym(2).declarator_ast;
  sym(1).pointer_ast = ast;
} break;
./

Declarator ::= DirectDeclarator ;

DirectDeclarator ::= T_ID ;
/.
case $rule_number: {
  IdentifierDeclaratorAST *ast = new IdentifierDeclaratorAST;
  ast->id = sym(1).string_ast;
  sym(1).identifier_declarator_ast = ast;
} break;
./

DirectDeclarator ::= DirectDeclarator T_LPAREN ArgumentsOpt T_RPAREN ;
/.
case $rule_number: {
  FunctionDeclaratorAST *ast = new FunctionDeclaratorAST;
  ast->declarator = sym(1).declarator_ast;
  ast->arguments = sym(3).arguments_ast;
  sym(1).function_declarator_ast = ast;
} break;
./

-- DirectDeclarator ::= DirectDeclarator T_LBRACKET T_NUM T_RBRACKET ;
-- /.
-- case $rule_number: {
--   ArrayDeclaratorAST *ast = new ArrayDeclaratorAST;
--   ast->declarator = sym(1).declarator_ast;
--   ast->dimension = sym(3).int_ast;
--   sym(1).array_declarator_ast = ast;
-- } break;
-- ./

DirectDeclarator ::= T_LPAREN Declarator T_RPAREN ;
/.
case $rule_number: {
  SubDeclaratorAST *ast = new SubDeclaratorAST;
  ast->declarator = sym(2).declarator_ast;
  sym(1).sub_declarator_ast = ast;
} break;
./

AbstractDeclaratorOpt ::= ;
/.
case $rule_number: {
  sym(1).declarator_ast = 0;
} break;
./

AbstractDeclaratorOpt ::= AbstractDeclarator ;

AbstractDeclarator ::= T_STAR AbstractDeclarator ;
/.
case $rule_number: {
  PointerAST *ast = new PointerAST;
  ast->declarator = sym(2).declarator_ast;
  sym(1).pointer_ast = ast;
} break;
./

AbstractDeclarator ::= T_STAR ;
/.
case $rule_number: {
  PointerAST *ast = new PointerAST;
  ast->declarator = 0;
  sym(1).pointer_ast = ast;
} break;
./

AbstractDeclarator ::= AbstractDirectDeclarator ;

AbstractDirectDeclarator ::= AbstractDirectDeclarator T_LPAREN ArgumentsOpt T_RPAREN ;
/.
case $rule_number: {
  FunctionDeclaratorAST *ast = new FunctionDeclaratorAST;
  ast->declarator = sym(1).declarator_ast;
  ast->arguments = sym(3).arguments_ast;
  sym(1).function_declarator_ast = ast;
} break;
./

-- AbstractDirectDeclarator ::= AbstractDirectDeclarator T_LBRACKET T_NUM T_RBRACKET ;
-- /.
-- case $rule_number: {
--   ArrayDeclaratorAST *ast = new ArrayDeclaratorAST;
--   ast->declarator = sym(1).declarator_ast;
--   ast->dimension = sym(3).int_ast;
--   sym(1).array_declarator_ast = ast;
-- } break;
-- ./

AbstractDirectDeclarator ::= T_LPAREN AbstractDeclarator T_RPAREN ;
/.
case $rule_number: {
  SubDeclaratorAST *ast = new SubDeclaratorAST;
  ast->declarator = sym(2).declarator_ast;
  sym(1).sub_declarator_ast = ast;
} break;
./

Expression ::= T_ID ;
/.
case $rule_number: {
  IdentifierAST *ast = new IdentifierAST;
  ast->id = sym(1).string_ast;
  sym(1).identifier_ast = ast;
} break;
./

Expression ::= T_STRING_LITERAL ;
/.
case $rule_number: {
  StringLiteralAST *ast = new StringLiteralAST;
  ast->value = sym(1).string_ast;
  sym(1).string_literal_ast = ast;
} break;
./

Expression ::= T_NUM ;
/.
case $rule_number: {
  IntConstantAST *ast = new IntConstantAST;
  ast->value = sym(1).int_ast;
  sym(1).int_constant_ast = ast;
} break;
./

Expression ::= T_LPAREN Expression T_RPAREN ;
/.
case $rule_number: {
  sym(1) = sym(2);
} break;
./

Expression ::= T_NOT Expression ;
/.
case $rule_number: {
  sym(1).unary_expression_ast = new UnaryExpressionAST(op_not, sym(2).expression_ast);
} break;
./

Expression ::= T_MINUS Expression ;
/.
case $rule_number: {
  sym(1).unary_expression_ast = new UnaryExpressionAST(op_neg, sym(2).expression_ast);
} break;
./

Expression ::= T_STAR Expression ;
/.
case $rule_number: {
  sym(1).unary_expression_ast = new UnaryExpressionAST(op_deref, sym(2).expression_ast);
} break;
./

Expression ::= T_AND Expression ;
/.
case $rule_number: {
  sym(1).unary_expression_ast = new UnaryExpressionAST(op_ref, sym(2).expression_ast);
} break;
./

Expression ::= T_LPAREN TypeSpec AbstractDeclaratorOpt T_RPAREN Expression ;
/.
case $rule_number: {
  CastExpressionAST *ast = new CastExpressionAST;
  ast->type_specifier = sym(2).string_ast;
  ast->declarator = sym(3).declarator_ast;
  ast->expression = sym(5).expression_ast;
  sym(1).cast_expression_ast = ast;
} break;
./

Expression ::= Expression T_LPAREN ExpressionListOpt T_RPAREN ;
/.
case $rule_number: {
  CallExpressionAST *ast = new CallExpressionAST;
  ast->expression = sym(1).expression_ast;
  ast->expression_list = sym(3).expression_list_ast;
  sym(1).call_expression_ast = ast;
} break;
./

Expression ::= Expression T_PLUS Expression ;
/.
case $rule_number: {
  sym(1).binary_expression_ast = new BinaryExpressionAST(op_add, sym(1).expression_ast, sym(3).expression_ast);
} break;
./

Expression ::= Expression T_MINUS Expression ;
/.
case $rule_number: {
  sym(1).binary_expression_ast = new BinaryExpressionAST(op_sub, sym(1).expression_ast, sym(3).expression_ast);
} break;
./

Expression ::= Expression T_STAR Expression ;
/.
case $rule_number: {
  sym(1).binary_expression_ast = new BinaryExpressionAST(op_mul, sym(1).expression_ast, sym(3).expression_ast);
} break;
./

Expression ::= Expression T_DIV Expression ;
/.
case $rule_number: {
  sym(1).binary_expression_ast = new BinaryExpressionAST(op_div, sym(1).expression_ast, sym(3).expression_ast);
} break;
./

Expression ::= Expression T_ASSIGN Expression ;
/.
case $rule_number: {
  sym(1).binary_expression_ast = new BinaryExpressionAST(op_assign, sym(1).expression_ast, sym(3).expression_ast);
} break;
./

Expression ::= Expression T_OR_OR Expression ;
/.
case $rule_number: {
  sym(1).binary_expression_ast = new BinaryExpressionAST(op_or, sym(1).expression_ast, sym(3).expression_ast);
} break;
./

Expression ::= Expression T_AND_AND Expression ;
/.
case $rule_number: {
  sym(1).binary_expression_ast = new BinaryExpressionAST(op_and, sym(1).expression_ast, sym(3).expression_ast);
} break;
./

Expression ::= Expression T_EQ_EQ Expression ;
/.
case $rule_number: {
  sym(1).binary_expression_ast = new BinaryExpressionAST(op_eq, sym(1).expression_ast, sym(3).expression_ast);
} break;
./

Expression ::= Expression T_NOT_EQ Expression ;
/.
case $rule_number: {
  sym(1).binary_expression_ast = new BinaryExpressionAST(op_not_eq, sym(1).expression_ast, sym(3).expression_ast);
} break;
./

Expression ::= Expression T_LT Expression ;
/.
case $rule_number: {
  sym(1).binary_expression_ast = new BinaryExpressionAST(op_lt, sym(1).expression_ast, sym(3).expression_ast);
} break;
./

Expression ::= Expression T_GT Expression ;
/.
case $rule_number: {
  sym(1).binary_expression_ast = new BinaryExpressionAST(op_gt, sym(1).expression_ast, sym(3).expression_ast);
} break;
./

Expression ::= Expression T_LE Expression ;
/.
case $rule_number: {
  sym(1).binary_expression_ast = new BinaryExpressionAST(op_le, sym(1).expression_ast, sym(3).expression_ast);
} break;
./

Expression ::= Expression T_GE Expression ;
/.
case $rule_number: {
  sym(1).binary_expression_ast = new BinaryExpressionAST(op_ge, sym(1).expression_ast, sym(3).expression_ast);
} break;
./

ExpressionListOpt ::= ;
/.
case $rule_number: {
  sym(1).expression_list_ast = 0;
} break;
./

ExpressionListOpt ::= ExpressionList ;

ExpressionList ::= Expression ;
/.
case $rule_number: {
  ExpressionListAST *ast = new ExpressionListAST;
  ast->push_back(sym(1).expression_ast);
  sym(1).expression_list_ast = ast;
} break;
./

ExpressionList ::= ExpressionList T_COMMA Expression ;
/.
case $rule_number: {
  sym(1).expression_list_ast->push_back(sym(3).expression_ast);
} break;
./



/.

} // switch

	  }

	else
	  break;
      }

    std::cerr << "*** Syntax Error at line " << yylineno << std::endl;

    return false;
  }
};

TranslationUnitAST *yyparse(const std::string &filename) {
  MiniCParser parser(filename);

  return parser.parse();
}

#define YY_DECL int MiniCParser::yylex()

#include "scanner.incl"


./
