

%option noyywrap

%%

"\n" {
  ++yylineno;
}

[ \t\r]+ {
  /* skip */
}

"//".* {
  /* skip */
}

"&" { return T_AND; }
"," { return T_COMMA; }
"/" { return T_DIV; }
"=" { return T_ASSIGN; }
">" { return T_GT; }
"{" { return T_LBRACE; }
"[" { return T_LBRACKET; }
"(" { return T_LPAREN; }
"<" { return T_LT; }
"-" { return T_MINUS; }
"!" { return T_NOT; }
"+" { return T_PLUS; }
"}" { return T_RBRACE; }
"]" { return T_RBRACKET; }
")" { return T_RPAREN; }
";" { return T_SEMICOLON; }
"*" { return T_STAR; }

"if" {
  return T_IF;
}

"else" {
  return T_ELSE;
}

"while" {
  return T_WHILE;
}

"return" {
  return T_RETURN;
}

"int" {
  return T_INT;
}

"char" {
  return T_CHAR;
}

"||" {
  return T_OR_OR;
}

"&&" {
  return T_AND_AND;
}

"==" {
  return T_EQ_EQ;
}

"!=" {
  return T_NOT_EQ;
}

"<=" {
  return T_LE;
}

">=" {
  return T_GE;
}

\"([^"]|\\\")*\" {
  yylval.string_ast = escape(std::string(yytext+1, yyleng - 2));
  return T_STRING_LITERAL;
}

'([^']|\\')*' {
  yylval.int_ast = (*escape(std::string(yytext+1, yyleng - 2)))[0];
  return T_NUM;
}

[a-zA-Z_][a-zA-Z_0-9]* {
  yylval.string_ast = intern(yytext);
  return T_ID;
}

0[xX][0-9a-fA-F]+ {
  yylval.int_ast = (int) strtod(yytext, 0);
  return T_NUM;
}

[0-9]+ {
  yylval.int_ast = (int) strtod(yytext, 0);
  return T_NUM;
}

. {
  return yytext[0];
}

%%

